@@ -65,6 +65,7 @@ from guake.globals import ALIGN_TOP
 from guake.globals import ALWAYS_ON_PRIMARY
 from guake.globals import MAX_TRANSPARENCY
 from guake.globals import NAME
+from guake.globals import TABS_SESSION_SCHEMA_VERSION
 from guake.gsettings import GSettingHandler
 from guake.guake_logging import setupLogging
 from guake.keybindings import Keybindings
@@ -118,6 +119,7 @@ class Guake(SimpleGladeApp):
     def __init__(self):
 
         def load_schema():
+            log.info("Loading Gnome schema from: %s", SCHEMA_DIR)
             return Gio.SettingsSchemaSource.new_from_directory(
                 SCHEMA_DIR, Gio.SettingsSchemaSource.get_default(), False
             )
@@ -130,6 +132,8 @@ class Guake(SimpleGladeApp):
             schema_source = load_schema()
         self.settings = Settings(schema_source)
 
+        log.info("Language previously loaded from: %s", LOCALE_DIR)
+
         super(Guake, self).__init__(gladefile('guake.glade'))
 
         select_gtk_theme(self.settings)
@@ -172,10 +176,19 @@ class Guake(SimpleGladeApp):
 
         # important widgets
         self.window = self.get_widget('window-root')
+        self.window.set_name('guake-terminal')
         self.window.set_keep_above(True)
         self.mainframe = self.get_widget('mainframe')
         self.mainframe.remove(self.get_widget('notebook-teminals'))
 
+        # Pending restore for terminal split after show-up
+        #     [(RootTerminalBox, TerminaBox, panes), ...]
+        self.pending_restore_page_split = []
+        self._failed_restore_page_split = []
+
+        # FullscreenManager
+        self.fullscreen_manager = FullscreenManager(self.settings, self.window, self)
+
         # Workspace tracking
         self.notebook_manager = NotebookManager(
             self.window, self.mainframe,
@@ -187,30 +200,8 @@ class Guake(SimpleGladeApp):
         self.set_tab_position()
 
         # check and set ARGB for real transparency
-        color = self.window.get_style_context().get_background_color(Gtk.StateFlags.NORMAL)
-        self.window.set_app_paintable(True)
-
-        def draw_callback(widget, cr):
-            if widget.transparency:
-                cr.set_source_rgba(color.red, color.green, color.blue, 1)
-            else:
-                cr.set_source_rgb(0, 0, 0)
-            cr.set_operator(cairo.OPERATOR_SOURCE)
-            cr.paint()
-            cr.set_operator(cairo.OPERATOR_OVER)
-
-        screen = self.window.get_screen()
-        visual = screen.get_rgba_visual()
-
-        if visual and screen.is_composited():
-            self.window.set_visual(visual)
-            self.window.transparency = True
-        else:
-            log.warn('System doesn\'t support transparency')
-            self.window.transparency = False
-            self.window.set_visual(screen.get_system_visual())
-
-        self.window.connect('draw', draw_callback)
+        self.update_visual()
+        self.window.get_screen().connect('composited-changed', self.update_visual)
 
         # Debounce accel_search_terminal
         self.prev_accel_search_terminal_time = 0.0
@@ -227,7 +218,7 @@ class Guake(SimpleGladeApp):
         # store the default window title to reset it when update is not wanted
         self.default_window_title = self.window.get_title()
 
-        self.abbreviate = False
+        self.display_tab_names = 0
 
         self.window.connect('focus-out-event', self.on_window_losefocus)
 
@@ -295,6 +286,25 @@ class Guake(SimpleGladeApp):
         # Tracking when reorder page
         notebook.connect('page-reordered', self.on_page_reorder)
 
+    def update_visual(self, user_data=None):
+        screen = self.window.get_screen()
+        visual = screen.get_rgba_visual()
+        if visual and screen.is_composited():
+            # NOTE: We should re-realize window when update window visual
+            # Otherwise it may failed, when the Guake it start without compositor
+            self.window.unrealize()
+            self.window.set_visual(visual)
+            self.window.set_app_paintable(True)
+            self.window.transparency = True
+            self.window.realize()
+            if self.window.get_property('visible'):
+                self.hide()
+                self.show()
+        else:
+            log.warn('System doesn\'t support transparency')
+            self.window.transparency = False
+            self.window.set_visual(screen.get_system_visual())
+
     # new color methods should be moved to the GuakeTerminal class
 
     def _load_palette(self):
@@ -321,8 +331,8 @@ class Guake(SimpleGladeApp):
             bg_color.alpha = 1
         return bg_color
 
-    def set_background_color_from_settings(self):
-        self.set_colors_from_settings()
+    def set_background_color_from_settings(self, terminal_uuid=None):
+        self.set_colors_from_settings(terminal_uuid)
 
     def get_bgcolor(self):
         palette_list = self._load_palette()
@@ -336,17 +346,65 @@ class Guake(SimpleGladeApp):
             font_color = Gdk.RGBA(0, 0, 0, 0)
         return font_color
 
-    def set_colors_from_settings(self):
+    def set_colors_from_settings(self, terminal_uuid=None):
         bg_color = self.get_bgcolor()
         font_color = self.get_fgcolor()
         palette_list = self._load_palette()
 
-        for i in self.get_notebook().iter_terminals():
+        terminals = self.get_notebook().iter_terminals()
+        if terminal_uuid:
+            terminals = [t for t in terminals if t.uuid == terminal_uuid]
+
+        for i in terminals:
             i.set_color_foreground(font_color)
             i.set_color_bold(font_color)
             i.set_colors(font_color, bg_color, palette_list[:16])
 
-    def set_bgcolor(self, bgcolor):
+    def set_colors_from_settings_on_page(self, current_terminal_only=False, page_num=None):
+        """If page_num is None, sets colors on the current page."""
+        bg_color = self.get_bgcolor()
+        font_color = self.get_fgcolor()
+        palette_list = self._load_palette()
+
+        if current_terminal_only:
+            terminal = self.get_notebook().get_current_terminal()
+            terminal.set_color_foreground(font_color)
+            terminal.set_color_bold(font_color)
+            terminal.set_colors(font_color, bg_color, palette_list[:16])
+        else:
+            if page_num is None:
+                page_num = self.get_notebook().get_current_page()
+            for terminal in self.get_notebook().get_nth_page(page_num).iter_terminals():
+                terminal.set_color_foreground(font_color)
+                terminal.set_color_bold(font_color)
+                terminal.set_colors(font_color, bg_color, palette_list[:16])
+
+    def reset_terminal_custom_colors(
+        self, current_terminal=False, current_page=False, terminal_uuid=None
+    ):
+        """Resets terminal(s) colors to the settings colors.
+        If current_terminal == False and current_page == False and terminal_uuid is None,
+        resets colors of all terminals.
+        """
+        terminals = []
+
+        if current_terminal:
+            terminals.append(self.get_notebook().get_current_terminal())
+        if current_page:
+            page_num = self.get_notebook().get_current_page()
+            for t in self.get_notebook().get_nth_page(page_num).iter_terminals():
+                terminals.append(t)
+        if terminal_uuid:
+            for t in self.get_notebook().iter_terminals():
+                if t.uuid == terminal_uuid:
+                    terminals.append(t)
+        if not current_terminal and not current_page and not terminal_uuid:
+            terminals = list(self.get_notebook().iter_terminals())
+
+        for i in terminals:
+            i.reset_custom_colors()
+
+    def set_bgcolor(self, bgcolor, current_terminal_only=False):
         if isinstance(bgcolor, str):
             c = Gdk.RGBA(0, 0, 0, 0)
             log.debug("Building Gdk Color from: %r", bgcolor)
@@ -356,11 +414,15 @@ class Guake(SimpleGladeApp):
             raise TypeError("color should be Gdk.RGBA, is: {!r}".format(bgcolor))
         bgcolor = self._apply_transparency_to_color(bgcolor)
         log.debug("setting background color to: %r", bgcolor)
-        page_num = self.get_notebook().get_current_page()
-        for terminal in self.get_notebook().get_nth_page(page_num).iter_terminals():
-            terminal.set_color_background(bgcolor)
 
-    def set_fgcolor(self, fgcolor):
+        if current_terminal_only:
+            self.get_notebook().get_current_terminal().set_color_background_custom(bgcolor)
+        else:
+            page_num = self.get_notebook().get_current_page()
+            for terminal in self.get_notebook().get_nth_page(page_num).iter_terminals():
+                terminal.set_color_background_custom(bgcolor)
+
+    def set_fgcolor(self, fgcolor, current_terminal_only=False):
         if isinstance(fgcolor, str):
             c = Gdk.RGBA(0, 0, 0, 0)
             log.debug("Building Gdk Color from: %r", fgcolor)
@@ -369,9 +431,13 @@ class Guake(SimpleGladeApp):
         if not isinstance(fgcolor, Gdk.RGBA):
             raise TypeError("color should be Gdk.RGBA, is: {!r}".format(fgcolor))
         log.debug("setting background color to: %r", fgcolor)
-        page_num = self.get_notebook().get_current_page()
-        for terminal in self.get_notebook().get_nth_page(page_num).iter_terminals():
-            terminal.set_color_foreground(fgcolor)
+
+        if current_terminal_only:
+            self.get_notebook().get_current_terminal().set_color_foreground_custom(fgcolor)
+        else:
+            page_num = self.get_notebook().get_current_page()
+            for terminal in self.get_notebook().get_nth_page(page_num).iter_terminals():
+                terminal.set_color_foreground_custom(fgcolor)
 
     def change_palette_name(self, palette_name):
         if isinstance(palette_name, str):
@@ -466,8 +532,9 @@ class Guake(SimpleGladeApp):
         return False
 
     def window_event(self, window, event):
-        state = event.new_window_state
-        log.debug("Received window state event: %s", state)
+        window_state = event.new_window_state
+        self.fullscreen_manager.set_window_state(window_state)
+        log.debug("Received window state event: %s", window_state)
 
     def show_hide(self, *args):
         """Toggles the main window visibility
@@ -565,6 +632,21 @@ class Guake(SimpleGladeApp):
             return True
         return False
 
+    def restore_pending_terminal_split(self):
+        # Restore pending terminal split
+        # XXX: Consider refactor how to handle failed restore page split
+        self.pending_restore_page_split = self._failed_restore_page_split
+        self._failed_restore_page_split = []
+        for root, box, panes in self.pending_restore_page_split:
+            if (
+                self.window.get_property('visible') and
+                root.get_notebook() == self.notebook_manager.get_current_notebook()
+            ):
+                root.restore_box_layout(box, panes)
+            else:
+                # Consider failed if the window is not visible
+                self._failed_restore_page_split.append((root, box, panes))
+
     def show(self):
         """Shows the main window and grabs the focus on it.
         """
@@ -581,7 +663,8 @@ class Guake(SimpleGladeApp):
             self.add_tab()
 
         self.window.set_keep_below(False)
-        self.window.show_all()
+        if not self.fullscreen_manager.is_fullscreen():
+            self.window.show_all()
         # this is needed because self.window.show_all() results in showing every
         # thing which includes the scrollbar too
         self.settings.general.triggerOnChangedValue(self.settings.general, "use-scrollbar")
@@ -591,7 +674,7 @@ class Guake(SimpleGladeApp):
         self.window.move(window_rect.x, window_rect.y)
 
         # this works around an issue in fluxbox
-        if not FullscreenManager(self.settings, self.window).is_fullscreen():
+        if not self.fullscreen_manager.is_fullscreen():
             self.settings.general.triggerOnChangedValue(self.settings.general, 'window-height')
 
         time = get_server_time(self.window)
@@ -637,7 +720,7 @@ class Guake(SimpleGladeApp):
         self.settings.styleBackground.triggerOnChangedValue(self.settings.styleBackground, 'color')
 
         log.debug("Current window position: %r", self.window.get_position())
-
+        self.restore_pending_terminal_split()
         self.execute_hook('show')
 
     def hide_from_remote(self):
@@ -667,6 +750,9 @@ class Guake(SimpleGladeApp):
         self.get_widget('window-root').unstick()
         self.window.hide()  # Don't use hide_all here!
 
+        # Hide popover
+        self.notebook_manager.get_current_notebook().popover.hide()
+
     def force_move_if_shown(self):
         if not self.hidden:
             # when displayed, GTK might refuse to move the window (X or Y position). Just hide and
@@ -678,42 +764,98 @@ class Guake(SimpleGladeApp):
 
     # -- configuration --
 
-    def load_config(self):
+    def load_config(self, terminal_uuid=None):
         """"Just a proxy for all the configuration stuff.
         """
+        user_data = {}
+        if terminal_uuid:
+            user_data['terminal_uuid'] = terminal_uuid
 
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'use-trayicon')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'prompt-on-quit')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'prompt-on-close-tab')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'window-tabbar')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'mouse-display')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'display-n')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'window-ontop')
-        if not FullscreenManager(self.settings, self.window).is_fullscreen():
-            self.settings.general.triggerOnChangedValue(self.settings.general, 'window-height')
-            self.settings.general.triggerOnChangedValue(self.settings.general, 'window-width')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'use-scrollbar')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'history-size')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'infinite-history')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'use-vte-titles')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'set-window-title')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'abbreviate-tab-names')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'max-tab-name-length')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'quick-open-enable')
         self.settings.general.triggerOnChangedValue(
-            self.settings.general, 'quick-open-command-line'
+            self.settings.general, 'use-trayicon', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'prompt-on-quit', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'prompt-on-close-tab', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'window-tabbar', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'fullscreen-hide-tabbar', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'mouse-display', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'display-n', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'window-ontop', user_data=user_data
+        )
+        if not self.fullscreen_manager.is_fullscreen():
+            self.settings.general.triggerOnChangedValue(
+                self.settings.general, 'window-height', user_data=user_data
+            )
+            self.settings.general.triggerOnChangedValue(
+                self.settings.general, 'window-width', user_data=user_data
+            )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'use-scrollbar', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'history-size', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'infinite-history', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'use-vte-titles', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'set-window-title', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'display-tab-names', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'max-tab-name-length', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'quick-open-enable', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'quick-open-command-line', user_data=user_data
+        )
+        self.settings.style.triggerOnChangedValue(
+            self.settings.style, 'cursor-shape', user_data=user_data
+        )
+        self.settings.styleFont.triggerOnChangedValue(
+            self.settings.styleFont, 'style', user_data=user_data
+        )
+        self.settings.styleFont.triggerOnChangedValue(
+            self.settings.styleFont, 'palette', user_data=user_data
+        )
+        self.settings.styleFont.triggerOnChangedValue(
+            self.settings.styleFont, 'palette-name', user_data=user_data
+        )
+        self.settings.styleFont.triggerOnChangedValue(
+            self.settings.styleFont, 'allow-bold', user_data=user_data
         )
-        self.settings.style.triggerOnChangedValue(self.settings.style, 'cursor-shape')
-        self.settings.styleFont.triggerOnChangedValue(self.settings.styleFont, 'style')
-        self.settings.styleFont.triggerOnChangedValue(self.settings.styleFont, 'palette')
-        self.settings.styleFont.triggerOnChangedValue(self.settings.styleFont, 'palette-name')
-        self.settings.styleFont.triggerOnChangedValue(self.settings.styleFont, 'allow-bold')
         self.settings.styleBackground.triggerOnChangedValue(
-            self.settings.styleBackground, 'transparency'
+            self.settings.styleBackground, 'transparency', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'use-default-font', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'compat-backspace', user_data=user_data
+        )
+        self.settings.general.triggerOnChangedValue(
+            self.settings.general, 'compat-delete', user_data=user_data
         )
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'use-default-font')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'compat-backspace')
-        self.settings.general.triggerOnChangedValue(self.settings.general, 'compat-delete')
 
     def accel_search_terminal(self, *args):
         nb = self.get_notebook()
@@ -795,7 +937,7 @@ class Guake(SimpleGladeApp):
         """Callback to increase transparency.
         """
         transparency = self.settings.styleBackground.get_int('transparency')
-        if int(transparency) - 2 > 0:
+        if int(transparency) > 0:
             self.settings.styleBackground.set_int('transparency', int(transparency) - 2)
         return True
 
@@ -803,7 +945,7 @@ class Guake(SimpleGladeApp):
         """Callback to decrease transparency.
         """
         transparency = self.settings.styleBackground.get_int('transparency')
-        if int(transparency) + 2 < MAX_TRANSPARENCY:
+        if int(transparency) < MAX_TRANSPARENCY:
             self.settings.styleBackground.set_int('transparency', int(transparency) + 2)
         return True
 
@@ -919,19 +1061,19 @@ class Guake(SimpleGladeApp):
         return True
 
     def accel_toggle_fullscreen(self, *args):
-        FullscreenManager(self.settings, self.window).toggle()
+        self.fullscreen_manager.toggle()
         return True
 
     def fullscreen(self):
-        FullscreenManager(self.settings, self.window).fullscreen()
+        self.fullscreen_manager.fullscreen()
 
     def unfullscreen(self):
-        FullscreenManager(self.settings, self.window).unfullscreen()
+        self.fullscreen_manager.unfullscreen()
 
     # -- callbacks --
 
     def recompute_tabs_titles(self):
-        """Updates labels on all tabs. This is required when `self.abbreviate`
+        """Updates labels on all tabs. This is required when `self.display_tab_names`
         changes
         """
         use_vte_titles = self.settings.general.get_boolean("use-vte-titles")
@@ -945,15 +1087,19 @@ class Guake(SimpleGladeApp):
             self.get_notebook().rename_page(page_num, self.compute_tab_title(terminal), False)
 
     def compute_tab_title(self, vte):
-        """Abbreviate and cut vte terminal title when necessary
+        """Compute the tab title
         """
         vte_title = vte.get_window_title() or _("Terminal")
         try:
             current_directory = vte.get_current_directory()
-            if self.abbreviate and vte_title.endswith(current_directory):
+            if self.display_tab_names == 1 and vte_title.endswith(current_directory):
                 parts = current_directory.split('/')
                 parts = [s[:1] for s in parts[:-1]] + [parts[-1]]
                 vte_title = vte_title[:len(vte_title) - len(current_directory)] + '/'.join(parts)
+            if self.display_tab_names == 2:
+                vte_title = current_directory.split('/')[-1]
+                if not vte_title:
+                    vte_title = "(root)"
         except OSError:
             pass
         return TabNameUtils.shorten(vte_title, self.settings)
@@ -961,6 +1107,8 @@ class Guake(SimpleGladeApp):
     @save_tabs_when_changed
     def on_terminal_title_changed(self, vte, term):
         # box must be a page
+        if not term.get_parent():
+            return
         box = term.get_parent().get_root_box()
         use_vte_titles = self.settings.general.get_boolean('use-vte-titles')
         if not use_vte_titles:
@@ -1015,14 +1163,19 @@ class Guake(SimpleGladeApp):
         self.get_notebook().rename_page(page_num, new_text, user_set)
 
     def terminal_spawned(self, notebook, terminal, pid):
-        self.load_config()
-        terminal.connect('window-title-changed', self.on_terminal_title_changed, terminal)
+        self.load_config(terminal_uuid=terminal.uuid)
+        terminal.handler_ids.append(
+            terminal.connect('window-title-changed', self.on_terminal_title_changed, terminal)
+        )
 
     @save_tabs_when_changed
     def add_tab(self, directory=None):
         """Adds a new tab to the terminal notebook.
         """
-        self.get_notebook().new_page_with_focus(directory)
+        position = None
+        if self.settings.general.get_boolean('new-tab-after'):
+            position = 1 + self.get_notebook().get_current_page()
+        self.get_notebook().new_page_with_focus(directory, position=position)
 
     def find_tab(self, directory=None):
         log.debug("find")
@@ -1080,10 +1233,8 @@ class Guake(SimpleGladeApp):
             self.set_terminal_focus()
 
         self.was_deleted_tab = True
-        abbreviate_tab_names = self.settings.general.get_boolean('abbreviate-tab-names')
-        if abbreviate_tab_names:
-            self.abbreviate = False
-            self.recompute_tabs_titles()
+        self.display_tab_names = self.settings.general.get_int('display-tab-names')
+        self.recompute_tabs_titles()
 
     def set_terminal_focus(self):
         """Grabs the focus on the current tab.
@@ -1151,15 +1302,22 @@ class Guake(SimpleGladeApp):
         return Path(xdg_config_home, 'guake').expanduser()
 
     def save_tabs(self, filename='session.json'):
-        config = {'schema_version': 1, 'timestamp': int(pytime.time()), 'workspace': {}}
+        config = {
+            'schema_version': TABS_SESSION_SCHEMA_VERSION,
+            'timestamp': int(pytime.time()),
+            'workspace': {}
+        }
+
         for key, nb in self.notebook_manager.get_notebooks().items():
             tabs = []
             for index in range(nb.get_n_pages()):
                 try:
                     page = nb.get_nth_page(index)
                     if page.child:
+                        panes = []
+                        page.save_box_layout(page.child, panes)
                         tabs.append({
-                            'directory': page.child.terminal.get_current_directory(),
+                            'panes': panes,
                             'label': nb.get_tab_text_index(index),
                             'custom_label_set': getattr(page, 'custom_label_set', False)
                         })
@@ -1180,7 +1338,7 @@ class Guake(SimpleGladeApp):
     def restore_tabs(self, filename='session.json', suppress_notify=False):
         session_file = self.get_xdg_config_directory() / filename
         if not session_file.exists():
-            log.info('Cannot found tabs session.json file')
+            log.info('Cannot find session.json file')
             return
         with session_file.open() as f:
             try:
@@ -1199,11 +1357,41 @@ class Guake(SimpleGladeApp):
                 )
                 return
 
+        # Check schema_version exist
+        if 'schema_version' not in config:
+            img_filename = pixmapfile('guake-notification.png')
+            notifier.showMessage(
+                _('Guake Terminal'),
+                _(
+                    'Tabs session restore abort.\n'
+                    'Your session file ({session_filename}) missing schema_version as key'
+                ).format(session_filename=session_file), img_filename
+            )
+            return
+
+        # Check schema version is not higher than current version
+        if config['schema_version'] > TABS_SESSION_SCHEMA_VERSION:
+            img_filename = pixmapfile('guake-notification.png')
+            notifier.showMessage(
+                _('Guake Terminal'),
+                _(
+                    'Tabs session restore abort.\n'
+                    'Your session file schema version is higher than current version '
+                    '({session_file_schema_version} > {current_schema_version}).'
+                ).format(
+                    session_file_schema_version=config['schema_version'],
+                    current_schema_version=TABS_SESSION_SCHEMA_VERSION
+                ), img_filename
+            )
+            return
+
         # Disable auto save tabs
         v = self.settings.general.get_boolean('save-tabs-when-changed')
         self.settings.general.set_boolean('save-tabs-when-changed', False)
 
         # Restore all tabs for all workspaces
+        self.pending_restore_page_split = []
+        self._failed_restore_page_split = []
         try:
             for key, frames in config['workspace'].items():
                 nb = self.notebook_manager.get_notebook(int(key))
@@ -1213,9 +1401,16 @@ class Guake(SimpleGladeApp):
                 # NOTE: If frame implement in future, we will need to update this code
                 for tabs in frames:
                     for index, tab in enumerate(tabs):
-                        nb.new_page_with_focus(
-                            tab['directory'], tab['label'], tab['custom_label_set']
+                        directory = tab['panes'][0]['directory'] if len(
+                            tab.get('panes', [])
+                        ) == 1 else tab.get('directory', None)
+                        box, page_num, term = nb.new_page_with_focus(
+                            directory, tab['label'], tab['custom_label_set']
                         )
+                        if tab.get('panes', False):
+                            # if directory:
+                            #     continue
+                            box.restore_box_layout(box.child, tab['panes'])
 
                     # Remove original pages in notebook
                     for i in range(current_pages):
