@@ -28,6 +28,7 @@ from guake.callbacks import NotebookScrollCallback
 from guake.dialogs import PromptQuitDialog
 from guake.menus import mk_notebook_context_menu
 from guake.prefs import PrefsDialog
+from guake.utils import gdk_is_x11_display
 from guake.utils import save_tabs_when_changed
 
 import gi
@@ -81,11 +82,24 @@ class TerminalNotebook(Gtk.Notebook):
             "button-press-event", self.on_button_press, None
         )
 
+        # Action box
         self.new_page_button = Gtk.Button(
             image=Gtk.Image.new_from_icon_name("tab-new", Gtk.IconSize.MENU), visible=True
         )
         self.new_page_button.connect("clicked", self.on_new_tab)
-        self.set_action_widget(self.new_page_button, Gtk.PackType.END)
+
+        self.tab_selection_button = Gtk.Button(
+            image=Gtk.Image.new_from_icon_name('pan-down-symbolic', Gtk.IconSize.MENU),
+            visible=True
+        )
+        self.popover = Gtk.Popover()
+        self.popover_window = None
+        self.tab_selection_button.connect('clicked', self.on_tab_selection)
+
+        self.action_box = Gtk.Box(visible=True)
+        self.action_box.pack_start(self.new_page_button, 0, 0, 0)
+        self.action_box.pack_end(self.tab_selection_button, 0, 0, 0)
+        self.set_action_widget(self.action_box, Gtk.PackType.END)
 
     def attach_guake(self, guake):
         self.guake = guake
@@ -106,6 +120,95 @@ class TerminalNotebook(Gtk.Notebook):
 
         return False
 
+    @save_tabs_when_changed
+    def on_new_tab(self, user_data):
+        self.new_page_with_focus()
+
+    def on_tab_selection(self, user_data):
+        """Construct the tab selection popover
+
+        Since we did not use Gtk.ListStore to store tab information, we will construct the
+        tab selection popover content each time when user click them.
+        """
+
+        # Remove previous window
+        if self.popover_window:
+            self.popover.remove(self.popover_window)
+
+        # This makes the list's background transparent
+        # ref: epiphany
+        css_provider = Gtk.CssProvider()
+        css_provider.load_from_data(
+            b'#popover-window list { border-style: none; background-color: transparent; }'
+        )
+        Gtk.StyleContext.add_provider_for_screen(
+            Gdk.Screen.get_default(), css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
+        )
+
+        # Construct popover properties
+        BOX_HEIGHT = 30
+        LISTBOX_MARGIN = 12
+        self.popover_window = Gtk.ScrolledWindow(name='popover-window')
+        self.popover_listbox = Gtk.ListBox()
+        self.popover_listbox.set_property('margin', LISTBOX_MARGIN)
+        self.popover_window.add_with_viewport(self.popover_listbox)
+
+        max_height = self.guake.window.get_allocation().height - BOX_HEIGHT if \
+            self.guake else BOX_HEIGHT * 10
+        height = BOX_HEIGHT * self.get_n_pages() + LISTBOX_MARGIN * 4
+        self.popover_window.set_min_content_height(min(max_height, height))
+        self.popover_window.set_min_content_width(325)
+        self.popover.add(self.popover_window)
+
+        # Construct content
+        current_term = self.get_current_terminal()
+        selected_row = 0
+        for i in range(self.get_n_pages()):
+            row = Gtk.ListBoxRow()
+            box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+            box.set_size_request(200, BOX_HEIGHT)
+            label = Gtk.Label(self.get_tab_text_index(i))
+            label.set_xalign(0.0)
+            box.pack_start(label, 0, 0, 5)
+            row.add(box)
+            setattr(row, 'page_index', i)
+            self.popover_listbox.add(row)
+            if current_term in self.get_terminals_for_page(i):
+                self.popover_listbox.select_row(row)
+                selected_row = i
+
+        # Signal
+        self.popover_listbox.connect('row-activated', self.on_popover_tab_select)
+
+        # Show popup
+        self.popover.set_position(Gtk.PositionType.TOP)
+        self.popover.set_relative_to(user_data)
+        self.popover.show_all()
+        try:
+            # GTK >= 3.22
+            self.popover.popup()
+        except AttributeError:
+            pass
+
+        # Adjust scrollor
+        while Gtk.events_pending():
+            Gtk.main_iteration()
+
+        if selected_row:
+            adj = self.popover_window.get_vadjustment()
+            v = adj.get_upper() - adj.get_page_size()
+            part = v / self.get_n_pages()
+            adj.set_value(part * (selected_row + 1))
+
+    def on_popover_tab_select(self, list_box, row):
+        page_index = getattr(row, 'page_index', -1)
+        if page_index != -1:
+            self.set_current_page(page_index)
+            self.get_terminals_for_page(page_index)[0].grab_focus()
+
+    def set_tabbar_visible(self, v):
+        self.set_property('show-tabs', v)
+
     def set_last_terminal_focused(self, terminal):
         self.last_terminal_focused = terminal
 
@@ -219,15 +322,22 @@ class TerminalNotebook(Gtk.Notebook):
     def delete_page_current(self, kill=True, prompt=0):
         self.delete_page(self.get_current_page(), kill, prompt)
 
-    def new_page(self, directory=None):
+    def new_page(self, directory=None, position=None):
         terminal = self.terminal_spawn(directory)
         terminal_box = TerminalBox()
         terminal_box.set_terminal(terminal)
         root_terminal_box = RootTerminalBox(self.guake, self)
         root_terminal_box.set_child(terminal_box)
-        page_num = self.append_page(root_terminal_box, None)
+        page_num = self.insert_page(
+            root_terminal_box, None, position if position is not None else -1
+        )
         self.set_tab_reorderable(root_terminal_box, True)
         self.show_all()  # needed to show newly added tabs and pages
+        # this is needed because self.window.show_all() results in showing every
+        # thing which includes the scrollbar too
+        self.guake.settings.general.triggerOnChangedValue(
+            self.guake.settings.general, "use-scrollbar"
+        )
         # this is needed to initially set the last_terminal_focused,
         # one could also call terminal.get_parent().on_terminal_focus()
         self.terminal_attached(terminal)
@@ -240,9 +350,14 @@ class TerminalNotebook(Gtk.Notebook):
             directory = os.environ['HOME']
             try:
                 if self.guake.settings.general.get_boolean('open-tab-cwd'):
-                    active_terminal = self.get_focused_terminal()
+                    # Do last focused terminal still alive?
+                    active_terminal = self.get_current_terminal()
+                    if not active_terminal:
+                        # If not alive, can we get any focused terminal?
+                        active_terminal = self.get_focused_terminal()
                     directory = os.path.expanduser('~')
                     if active_terminal:
+                        # If found, we will use its directory as new terminal's directory
                         directory = active_terminal.get_current_directory()
             except Exception as e:
                 pass
@@ -254,14 +369,15 @@ class TerminalNotebook(Gtk.Notebook):
         terminal.emit("focus", Gtk.DirectionType.TAB_FORWARD)
         self.emit('terminal-spawned', terminal, terminal.pid)
 
-    def new_page_with_focus(self, directory=None, label=None, user_set=False):
-        box, page_num, terminal = self.new_page(directory)
+    def new_page_with_focus(self, directory=None, label=None, user_set=False, position=None):
+        box, page_num, terminal = self.new_page(directory, position=position)
         self.set_current_page(page_num)
         if not label:
             self.rename_page(page_num, _("Terminal"), False)
         else:
             self.rename_page(page_num, label, user_set)
         terminal.grab_focus()
+        return box, page_num, terminal
 
     def rename_page(self, page_index, new_text, user_set=False):
         """Rename an already added page by its index. Use user_set to define
@@ -301,10 +417,6 @@ class TerminalNotebook(Gtk.Notebook):
     def get_tab_text_page(self, page):
         return self.get_tab_label(page).get_text()
 
-    @save_tabs_when_changed
-    def on_new_tab(self, user_data):
-        self.new_page_with_focus()
-
     def on_show_preferences(self, user_data):
         self.guake.hide()
         PrefsDialog(self.guake.settings).show()
@@ -337,9 +449,9 @@ class TerminalNotebook(Gtk.Notebook):
         dialog.show()
 
     def restore_tabs_dialog_response(self, widget, response_id):
+        widget.destroy()
         if response_id == Gtk.ResponseType.OK:
             self.guake.restore_tabs()
-        widget.destroy()
 
 
 class NotebookManager(GObject.Object):
@@ -359,11 +471,15 @@ class NotebookManager(GObject.Object):
         self.notebook_parent = notebook_parent
         self.terminal_spawned_cb = terminal_spawned_cb
         self.page_deleted_cb = page_deleted_cb
-        self.screen = Wnck.Screen.get_default()
-        if workspaces_enabled:
-            self.screen.connect("active-workspace-changed", self.workspace_changed)
+        if workspaces_enabled and gdk_is_x11_display(Gdk.Display.get_default()):
+            # NOTE: Wnck didn't support non-X11 display backend, so we need to check if the display
+            #       is X11 or not, if not, it will not able to enable workspace-specific-tab-sets
+            #
+            # TODO: Is there anyway to support this in non-X11 display backend?
+            self.screen = Wnck.Screen.get_default()
+            self.screen.connect("active-workspace-changed", self.__workspace_changed_cb)
 
-    def workspace_changed(self, screen, previous_workspace):
+    def __workspace_changed_cb(self, screen, previous_workspace):
         self.set_workspace(self.screen.get_active_workspace().get_number())
 
     def get_notebook(self, workspace_index: int):
@@ -396,6 +512,13 @@ class NotebookManager(GObject.Object):
                 notebook.last_terminal_focused is not None:
             notebook.last_terminal_focused.grab_focus()
 
+        # Restore pending page terminal split
+        notebook.guake.restore_pending_terminal_split()
+
+    def set_notebooks_tabbar_visible(self, v):
+        for nb in self.iter_notebooks():
+            nb.set_tabbar_visible(v)
+
     def get_notebooks(self):
         return self.notebooks
 
@@ -410,6 +533,12 @@ class NotebookManager(GObject.Object):
             for t in self.notebooks[k].iter_terminals():
                 yield t
 
+    def get_terminal_by_uuid(self, terminal_uuid):
+        for t in self.iter_terminals():
+            if t.uuid == terminal_uuid:
+                return t
+        return None
+
     def iter_pages(self):
         for k in self.notebooks:
             for t in self.notebooks[k].iter_pages():
