@@ -90,32 +90,28 @@ HOTKEYS = [
     {
         'label': _('General'),
         'key': 'general',
-        'keys': [
-            {
-                'key': 'show-hide',
-                'label': _('Toggle Guake visibility')
-            },
-            {
-                'key': 'show-focus',
-                'label': _('Show and focus Guake window')
-            },
-            {
-                'key': 'toggle-fullscreen',
-                'label': _('Toggle Fullscreen')
-            },
-            {
-                'key': 'toggle-hide-on-lose-focus',
-                'label': _('Toggle Hide on Lose Focus')
-            },
-            {
-                'key': 'quit',
-                'label': _('Quit')
-            },
-            {
-                'key': 'reset-terminal',
-                'label': _('Reset terminal')
-            },
-        ]
+        'keys': [{
+            'key': 'show-hide',
+            'label': _('Toggle Guake visibility')
+        }, {
+            'key': 'show-focus',
+            'label': _('Show and focus Guake window')
+        }, {
+            'key': 'toggle-fullscreen',
+            'label': _('Toggle Fullscreen')
+        }, {
+            'key': 'toggle-hide-on-lose-focus',
+            'label': _('Toggle Hide on Lose Focus')
+        }, {
+            'key': 'quit',
+            'label': _('Quit')
+        }, {
+            'key': 'reset-terminal',
+            'label': _('Reset terminal')
+        }, {
+            'key': 'search-terminal',
+            'label': _('Search terminal')
+        }]
     },
     {
         'label': _('Tab management'),
@@ -354,6 +350,21 @@ class PrefsCallbacks():
 
     # general tab
 
+    def on_restore_tabs_startup_toggled(self, chk):
+        """Changes the activity of restore-tabs-startup in dconf
+        """
+        self.settings.general.set_boolean('restore-tabs-startup', chk.get_active())
+
+    def on_restore_tabs_notify_toggled(self, chk):
+        """Changes the activity of restore-tabs-notify in dconf
+        """
+        self.settings.general.set_boolean('restore-tabs-notify', chk.get_active())
+
+    def on_save_tabs_when_changed_toggled(self, chk):
+        """Changes the activity of save-tabs-when-changed in dconf
+        """
+        self.settings.general.set_boolean('save-tabs-when-changed', chk.get_active())
+
     def on_default_shell_changed(self, combo):
         """Changes the activity of default_shell in dconf
         """
@@ -429,6 +440,11 @@ class PrefsCallbacks():
         """
         self.settings.general.set_boolean('tab-ontop', chk.get_active())
 
+    def on_new_tab_after_toggled(self, chk):
+        """Changes the activity of new_tab_after in dconf
+        """
+        self.settings.general.set_boolean('new-tab-after', chk.get_active())
+
     def on_quick_open_enable_toggled(self, chk):
         """Changes the activity of quick_open_enable in dconf
         """
@@ -461,6 +477,11 @@ class PrefsCallbacks():
         """
         self.settings.general.set_boolean('window-tabbar', chk.get_active())
 
+    def on_fullscreen_hide_tabbar_toggled(self, chk):
+        """Changes the activity of fullscreen_hide_tabbar in dconf
+        """
+        self.settings.general.set_boolean('fullscreen-hide-tabbar', chk.get_active())
+
     def on_start_fullscreen_toggled(self, chk):
         """Changes the activity of start_fullscreen in dconf
         """
@@ -482,10 +503,10 @@ class PrefsCallbacks():
         """
         self.settings.general.set_boolean('set-window-title', chk.get_active())
 
-    def on_abbreviate_tab_names_toggled(self, chk):
-        """Save `abbreviate_tab_names` property value in dconf
+    def on_tab_name_display_changed(self, combo):
+        """Save `display-tab-names` property value in dconf
         """
-        self.settings.general.set_boolean('abbreviate-tab-names', chk.get_active())
+        self.settings.general.set_int('display-tab-names', combo.get_active())
 
     def on_max_tab_name_length_changed(self, spin):
         """Changes the value of max_tab_name_length in dconf
@@ -879,10 +900,10 @@ class PrefsDialog(SimpleGladeApp):
 
     def update_vte_subwidgets_states(self):
         do_use_vte_titles = self.get_widget('use_vte_titles').get_active()
-        max_tab_name_length_wdg = self.get_widget('max_tab_name_length')
-        max_tab_name_length_wdg.set_sensitive(do_use_vte_titles)
+        self.get_widget('tab_name_display').set_sensitive(do_use_vte_titles)
+        self.get_widget('lbl_tab_name_display').set_sensitive(do_use_vte_titles)
+        self.get_widget('max_tab_name_length').set_sensitive(do_use_vte_titles)
         self.get_widget('lbl_max_tab_name_length').set_sensitive(do_use_vte_titles)
-        self.get_widget('abbreviate_tab_names').set_sensitive(do_use_vte_titles)
 
     def on_reset_compat_defaults_clicked(self, bnt):
         """Reset default values to compat_{backspace,delete} dconf
@@ -1065,6 +1086,18 @@ class PrefsDialog(SimpleGladeApp):
         """
         self._load_default_shell_settings()
 
+        # restore tabs startup
+        value = self.settings.general.get_boolean('restore-tabs-startup')
+        self.get_widget('restore-tabs-startup').set_active(value)
+
+        # restore tabs notify
+        value = self.settings.general.get_boolean('restore-tabs-notify')
+        self.get_widget('restore-tabs-notify').set_active(value)
+
+        # save tabs when changed
+        value = self.settings.general.get_boolean('save-tabs-when-changed')
+        self.get_widget('save-tabs-when-changed').set_active(value)
+
         # login shell
         value = self.settings.general.get_boolean('use-login-shell')
         self.get_widget('use_login_shell').set_active(value)
@@ -1126,10 +1159,10 @@ class PrefsDialog(SimpleGladeApp):
         value = self.settings.general.get_boolean('set-window-title')
         self.get_widget('set_window_title').set_active(value)
 
-        # abbreviate tab names
-        self.get_widget('abbreviate_tab_names').set_sensitive(value)
-        value = self.settings.general.get_boolean('abbreviate-tab-names')
-        self.get_widget('abbreviate_tab_names').set_active(value)
+        # set tab name display method
+        self.get_widget('tab_name_display').set_sensitive(value)
+        value = self.settings.general.get_int('display-tab-names')
+        self.get_widget('tab_name_display').set_active(value)
 
         # max tab name length
         value = self.settings.general.get_int('max-tab-name-length')
@@ -1166,6 +1199,10 @@ class PrefsDialog(SimpleGladeApp):
         value = self.settings.general.get_boolean('window-tabbar')
         self.get_widget('window_tabbar').set_active(value)
 
+        # fullscreen hide tabbar
+        value = self.settings.general.get_boolean('fullscreen-hide-tabbar')
+        self.get_widget('fullscreen_hide_tabbar').set_active(value)
+
         # start fullscreen
         value = self.settings.general.get_boolean('start-fullscreen')
         self.get_widget('start_fullscreen').set_active(value)
@@ -1398,7 +1435,7 @@ class PrefsDialog(SimpleGladeApp):
                     "Please try with a key such as "
                     "Control, Alt or Shift at the same "
                     "time.\n"
-                ) % html_escape(key)
+                ) % html_escape(chr(key))
             )
             dialog.run()
             dialog.destroy()
