@@ -72,7 +72,7 @@ except Exception as e:
     sys.stderr.write(
         "[WARN] Some feature might not work:\n"
         "[WARN]  - 'exit' command might freeze the terminal instead of closing the tab\n"
-        "[WARN]  - the 'wall' command is know to work badly\n"
+        "[WARN]  - the 'wall' command is known to work badly\n"
     )
     sys.stderr.write("[WARN] Error: " + str(e) + '\n')
     sys.stderr.write(
@@ -110,11 +110,16 @@ class GuakeTerminal(Vte.Terminal):
         self.matched_value = ''
         self.font_scale_index = 0
         self._pid = None
-        self.custom_bgcolor = None
-        self.custom_fgcolor = None
+        # self.custom_bgcolor = None
+        # self.custom_fgcolor = None
         self.found_link = None
         self.uuid = uuid.uuid4()
 
+        # Custom colors
+        self.custom_bgcolor = None
+        self.custom_fgcolor = None
+        self.custom_palette = None
+
         self.setup_drag_and_drop()
 
     def setup_drag_and_drop(self):
@@ -549,3 +554,83 @@ class GuakeTerminal(Vte.Terminal):
             libutempter.utempter_add_record(self.get_pty().get_fd(), os.uname()[1])
         self.pid = pid
         return pid
+
+    def set_color_foreground(self, font_color, *args, **kwargs):
+        real_fgcolor = self.custom_fgcolor if self.custom_fgcolor else font_color
+        super(GuakeTerminal, self).set_color_foreground(real_fgcolor, *args, **kwargs)
+
+    def set_color_background(self, bgcolor, *args, **kwargs):
+        real_bgcolor = self.custom_bgcolor if self.custom_bgcolor else bgcolor
+        super(GuakeTerminal, self).set_color_background(real_bgcolor, *args, **kwargs)
+
+    def set_color_bold(self, font_color, *args, **kwargs):
+        real_fgcolor = self.custom_fgcolor if self.custom_fgcolor else font_color
+        super(GuakeTerminal, self).set_color_bold(real_fgcolor, *args, **kwargs)
+
+    def set_colors(self, font_color, bg_color, palette_list, *args, **kwargs):
+        real_bgcolor = self.custom_bgcolor if self.custom_bgcolor else bg_color
+        real_fgcolor = self.custom_fgcolor if self.custom_fgcolor else font_color
+        real_palette = self.custom_palette if self.custom_palette else palette_list
+        super(GuakeTerminal,
+              self).set_colors(real_fgcolor, real_bgcolor, real_palette, *args, **kwargs)
+
+    def set_color_foreground_custom(self, fgcolor, *args, **kwargs):
+        """Sets custom foreground color for this terminal"""
+        print('set_color_foreground_custom: %s' % self.uuid)
+        self.custom_fgcolor = fgcolor
+        super(GuakeTerminal, self).set_color_foreground(self.custom_fgcolor, *args, **kwargs)
+
+    def set_color_background_custom(self, bgcolor, *args, **kwargs):
+        """Sets custom background color for this terminal"""
+        self.custom_bgcolor = bgcolor
+        super(GuakeTerminal, self).set_color_background(self.custom_bgcolor, *args, **kwargs)
+
+    def reset_custom_colors(self):
+        self.custom_fgcolor = None
+        self.custom_bgcolor = None
+        self.custom_palette = None
+
+    @staticmethod
+    def _color_to_list(color):
+        """This method is used for serialization."""
+        if color is None:
+            return None
+        return [color.red, color.green, color.blue, color.alpha]
+
+    @staticmethod
+    def _color_from_list(color_list):
+        """This method is used for deserialization."""
+        return Gdk.RGBA(
+            red=color_list[0], green=color_list[1], blue=color_list[2], alpha=color_list[3]
+        )
+
+    def get_custom_colors_dict(self):
+        """Returns dictionary of custom colors."""
+        return {
+            'fg_color': self._color_to_list(self.custom_fgcolor),
+            'bg_color': self._color_to_list(self.custom_bgcolor),
+            'palette': [self._color_to_list(col)
+                        for col in self.custom_palette] if self.custom_palette else None,
+        }
+
+    def set_custom_colors_from_dict(self, colors_dict):
+        if not isinstance(colors_dict, dict):
+            return
+
+        bg_color = colors_dict.get('bg_color', None)
+        if isinstance(bg_color, list):
+            self.custom_bgcolor = self._color_from_list(bg_color)
+        else:
+            self.custom_bgcolor = None
+
+        fg_color = colors_dict.get('fg_color', None)
+        if isinstance(fg_color, list):
+            self.custom_fgcolor = self._color_from_list(fg_color)
+        else:
+            self.custom_fgcolor = None
+
+        palette = colors_dict.get('palette', None)
+        if isinstance(palette, list):
+            self.custom_palette = [self._color_from_list(col) for col in palette]
+        else:
+            self.custom_palette = None
